import { Schema, model } from 'mongoose'

const schema = new Schema({
    name: String,
    items: [{ type: Schema.Types.ObjectId, required: true, ref: 'Item' }],
    created: { type: Date, default: () => new Date().toISOString() },
    // createdBy: { type: Schema.Types.ObjectId, required: true, ref: 'User' }
    deleted: { type: Boolean, default: false }
})

schema.virtual('id').get(function () {
    return this._id.toHexString()
})

schema.set('toJSON', {
    virtuals: true,
    versionKey: false,
    transform: (doc, ret) => {
        delete ret._id
    }
})

schema.method('addItem', async function(item) {
    this.items.push(item._id)
    return await this.save()
})

schema.method('removeItem', async function(item) {
    this.items = this.items.filter(i => i !== item._id)
    return await this.save()
})


export default model('TodoList', schema)
